==> abasic_0_0_3_5.zip: abasic_0_0_3_5/common.cpp <==
/*
 * aBasic
 * Copyright (C) 2007 m_inaba
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


//Shift_jis̗ӓ_
//Shift_jis1oCgڂƂ肤͈́@0x81~0x9F,E0~FC
//Shift_jis2oCgڂƂ肤͈́@0x40~0xFC

#include "common.h"
#include "vari.h"

//̕
void _strcpy(char*out,const char*in){
	char*start=out;
	while(*out++=*in++){
		if((out-start)>=MAX_STR_LEN){
			*out='\0';
			break;
		}
	}
}

//̎w蕶ׂĎ菜@SĂ̕菜܂
void trim_a(char*str,char c){
	if(c==*str){
		_strcpy(str,str+1);
	}else str++;
	if(*str) trim_a(str,c);
}

//̃Xy[XׂĎ菜@_uNI[e[Vň͂܂ꂽ͖܂
void trim_a(char*str){
	if(!str) return;
	if('\"'==*str){//""ň͂܂ꂽ͏O
		while('\"'!=*++str){
			if('\0'==*str) return;//"̖G[
			if('\\'==*str){
				str++;//GXP[vV[PX̓ǂݍ
				if('\0'==*str)return;//"̖G[
			}
		}
	}
	if(' '==*str){
		_strcpy(str,str+1);
	}else str++;
	if(*str) trim_a(str);
}

//̉ẼXy[Xƃ^u菜
void trim_r(char*str){
	char*i=str;
	while(*++i);
	while(str<--i && (' '==*i || '\t'==*i)) *i='\0';
}

//̍̃Xy[Xƃ^u菜
void trim_l(char*str){
	if(' '==*str || '\t'==*str) {
		_strcpy(str,str+1);
		trim_l(str);
	}
}

//^uXy[XɕϊA
//AXy[X̃Xy[Xɂ
void trim_c(char*str){
	if('\t'==*str) *str=' ';
	while(*str){
		if('\t'==*(str+1)) *(str+1)=' ';
		if((' '==*str)&&(' '==*(str+1))){
			_strcpy(str,str+1);	
		}else str++;
	}
}

//[̃Xy[Xƃ^u菜
void trim_rl(char*str){
	trim_r(str);
	trim_l(str);
}

//̐`@[̃Xy[X菜AAXy[X̃Xy[Xɂ
void trim_rlc(char*str){
	trim_rl(str);
	trim_c(str);
}

//̃JEg
int count_c(const char*str,const char c){
	int i=0;
	do{
		if(*str==c) i++;
	}while(*str++);
	return i;
}

//̐؂o@_uNI[e[Vň͂܂ꂽ͖܂
//strɐ؂o̕AcɃf~^ݒ肵܂B
//strNULLČĂяoƂɂAɐ؂oꂽĂяo܂B
char*_strchop(char*str,const char c){
	static char*pos;
	if(str){
		if('\0'==*str){
			pos=NULL;
			goto exit;
		}
		pos=str;
		while(*str){
			if('\"'==*str){//""ň͂܂ꂽ͏O
				while('\"'!=*++str){
					if('\0'==*str) goto exit;//"̖G[
#ifdef SHIFT_JIS
					//VtgJIS̓ǂݍ
					if((0x81<=(unsigned char)*str && 0x9F>=(unsigned char)*str) || (0xE0<=(unsigned char)*str && 0xFC>=(unsigned char)*str)){
						str++;//GXP[vV[PX̓ǂݍ
						if('\0'==*str) goto exit;//"̖G[
					}else 
#endif
					if('\\'==*str){
						str++;//GXP[vV[PX̓ǂݍ
						if('\0'==*str) goto exit;//"̖G[
					}
				}
			}
			if(c==*str) *str='\0';
			str++;
		}
		*++str=EOF;//̏I[
	}else{
		while(*pos++);
		if(EOF==*pos) pos=NULL;//̏I[̂NULLԂ
	}
exit:;
	return pos;
}

//̒u SĂ̕u܂
//buff:ƃobt@ str:Ώە a: b:u
//ǔʂ͖߂l̃|C^
char*replace(char*buff,char*str,const char*a,const char*b){
	char*ptr=str;
	char*st_ptr=str;
	if(!*a) return NULL;
	*buff='\0';
	while((ptr=strstr(st_ptr,a)) && (ptr)){
		*ptr='\0';
		strcat(buff,st_ptr);
		strcat(buff,b);
		st_ptr=ptr+strlen(a);
	}
	strcat(buff,st_ptr);
	return buff;
}


//ChJ[h܂ޕ
//pat:* ?܂ރp^[
//str:
//len:ꂽ̒
//߂l:ꂽ̐擪AhX łȂƂNULL
char*match(const char *pat,const char *str,int*len)
{
	char*pat_pos;
	char*str_pos;
	pat_pos=(char*)pat;
	str_pos=(char*)str;
	do{
		if('*'==*pat_pos){
			pat_pos++;
			while(*str_pos){
				if(*str_pos==*pat_pos){
					break;
				}else{
					str_pos++;
				}
			}
		}
		if(!*pat_pos){
			*len=(int)(str_pos-str);
			return (char*)str;
		}
		if(*str_pos==*pat_pos || '?'==*pat_pos){
			pat_pos++;
		}else{
			pat_pos=(char*)pat;
			str_pos=(char*)str++;
		}
	}while(*str_pos++);
	return NULL;
}


//ߕ̕
void instcpy(char*out,char*in){
	while(' '==*in || '\t'==*in) in++;//ŏ̃Xy[X菜
	while(*out=*in){
		if('\r'==*out || '\n'==*out || EOF==*out){//EOFAsR[h𖳎
			*out='\0';
			break;
		}

		//""ň͂܂ꂽ͏O
		if('\"'==*in){
			*out++=*in++;
			while('\"'!=*in){
				if('\0'==*in){*out='\0';return;}//"̖G[
#ifdef SHIFT_JIS
				//VtgJIS̓ǂݍ
				if((0x81<=(unsigned char)*in && 0x9F>=(unsigned char)*in) || (0xE0<=(unsigned char)*in && 0xFC>=(unsigned char)*in)){
					*out++=*in++;
					if('\0'==*in){*out='\0';break;}//vIȃG[
					*out++=*in++;
					if('\0'==*in){*out='\0';break;}//vIȃG[
				}else 
#endif
				if('\\'==*in){
					*out++=*in++;//GXP[vV[PX̓ǂݍ
					if('\0'==*in){*out='\0';return;}//"̖G[
				}else{
					*out++=*in++;
				}
			}
			*out=*in;
		}

		if('\''==*out){//'̓Rgƍl
			*out='\0';
			break;
		}

		if('\t'==*out){//tabXy[Xɒu
			*out=' ';
		}
		if((' '==*in || '\t'==*in) && (' '==*(in+1) || '\t'==*(in+1)))  out--;//AȂXy[X폜

		out++;in++;
	}
}


//s
void shell(char*str){

	str++;//擪菜
	trim_a(str,'\"');// "菜

#ifdef _WIN32 //gݍ݃}NɂVCG++̐؂蕪

//Windows
	STARTUPINFO si;// X^[gAbv
	PROCESS_INFORMATION pi;// vZX

	//STARTUPINFO \̂̓e擾 
	GetStartupInfo(&si);

	//ȂԂŋNɂ́A
	//si.dwFlags = STARTF_USESHOWWINDOW;
	//si.wShowWindow = SW_HIDE;

	if(0!=CreateProcess(
			NULL,					// s\W[̖O
			str,					// R}hC̕
			NULL,					// ZLeBLqq
			NULL,					// ZLeBLqq
			FALSE,					// nȟpIvV
			NULL,					// 쐬̃tO 
									// CREATE_NEW_PROCESS_GROUP	: VȃvZX
			NULL,					// VubN
			NULL,					// JgfBNg̖O
			&si,					// X^[gAbv
			&pi						// vZX
			)){

		//vZX̏Iҋ@
		CloseHandle(pi.hThread);
		WaitForSingleObject(pi.hProcess,INFINITE);
		CloseHandle(pi.hProcess);
	}else{
		//G[
		setVariable("system.err","ERR");
	}

#else
//Linux
	system(str);
#endif
}

//-------------------------------------------------------------------------------

==> abasic_0_0_3_6.zip: abasic_0_0_3_6/common.cpp <==
/*
 * aBasic
 * Copyright (C) 2007 m_inaba
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


//Shift_jis̗ӓ_
//Shift_jis1oCgڂƂ肤͈́@0x81~0x9F,E0~FC
//Shift_jis2oCgڂƂ肤͈́@0x40~0xFC

#include "common.h"
#include "vari.h"

//̕
void _strcpy(char*out,const char*in){
	char*start=out;
	while(*out++=*in++){
		if((out-start)>=MAX_STR_LEN){
			*out='\0';
			break;
		}
	}
}

//̎w蕶ׂĎ菜@SĂ̕菜܂
void trim_a(char*str,char c){
	if(c==*str){
		_strcpy(str,str+1);
	}else str++;
	if(*str) trim_a(str,c);
}

//̃Xy[XׂĎ菜@_uNI[e[Vň͂܂ꂽ͖܂
void trim_a(char*str){
	if(!str) return;
	if('\"'==*str){//""ň͂܂ꂽ͏O
		while('\"'!=*++str){
			if('\0'==*str) return;//"̖G[
			if('\\'==*str){
				str++;//GXP[vV[PX̓ǂݍ
				if('\0'==*str)return;//"̖G[
			}
		}
	}
	if(' '==*str){
		_strcpy(str,str+1);
	}else str++;
	if(*str) trim_a(str);
}

//̉ẼXy[Xƃ^u菜
void trim_r(char*str){
	char*i=str;
	while(*++i);
	while(str<--i && (' '==*i || '\t'==*i)) *i='\0';
}

//̍̃Xy[Xƃ^u菜
void trim_l(char*str){
	if(' '==*str || '\t'==*str) {
		_strcpy(str,str+1);
		trim_l(str);
	}
}

//^uXy[XɕϊA
//AXy[X̃Xy[Xɂ
void trim_c(char*str){
	if('\t'==*str) *str=' ';
	while(*str){
		if('\t'==*(str+1)) *(str+1)=' ';
		if((' '==*str)&&(' '==*(str+1))){
			_strcpy(str,str+1);	
		}else str++;
	}
}

//[̃Xy[Xƃ^u菜
void trim_rl(char*str){
	trim_r(str);
	trim_l(str);
}

//̐`@[̃Xy[X菜AAXy[X̃Xy[Xɂ
void trim_rlc(char*str){
	trim_rl(str);
	trim_c(str);
}

//̃JEg
int count_c(const char*str,const char c){
	int i=0;
	do{
		if(*str==c) i++;
	}while(*str++);
	return i;
}

//̐؂o@_uNI[e[Vň͂܂ꂽ͖܂
//strɐ؂o̕AcɃf~^ݒ肵܂B
//strNULLČĂяoƂɂAɐ؂oꂽĂяo܂B
char*_strchop(char*str,const char c){
	static char*pos;
	if(str){
		if('\0'==*str){
			pos=NULL;
			goto exit;
		}
		pos=str;
		while(*str){
			if('\"'==*str){//""ň͂܂ꂽ͏O
				while('\"'!=*++str){
					if('\0'==*str) goto exit;//"̖G[
#ifdef SHIFT_JIS
					//VtgJIS̓ǂݍ
					if((0x81<=(unsigned char)*str && 0x9F>=(unsigned char)*str) || (0xE0<=(unsigned char)*str && 0xFC>=(unsigned char)*str)){
						str++;//GXP[vV[PX̓ǂݍ
						if('\0'==*str) goto exit;//"̖G[
					}else 
#endif
					if('\\'==*str){
						str++;//GXP[vV[PX̓ǂݍ
						if('\0'==*str) goto exit;//"̖G[
					}
				}
			}
			if(c==*str) *str='\0';
			str++;
		}
		*++str=EOF;//̏I[
	}else{
		while(*pos++);
		if(EOF==*pos) pos=NULL;//̏I[̂NULLԂ
	}
exit:;
	return pos;
}

//̒u SĂ̕u܂
//buff:ƃobt@ str:Ώە a: b:u
//ǔʂ͖߂l̃|C^
char*replace(char*buff,char*str,const char*a,const char*b){
	char*ptr=str;
	char*st_ptr=str;
	if(!*a) return NULL;
	*buff='\0';
	while((ptr=strstr(st_ptr,a)) && (ptr)){
		*ptr='\0';
		strcat(buff,st_ptr);
		strcat(buff,b);
		st_ptr=ptr+strlen(a);
	}
	strcat(buff,st_ptr);
	return buff;
}


//ChJ[h܂ޕ
//pat:* ?܂ރp^[
//str:
//len:ꂽ̒
//߂l:ꂽ̐擪AhX łȂƂNULL
char*match(const char *pat,const char *str,int*len)
{
	char*pat_pos;
	char*str_pos;
	pat_pos=(char*)pat;
	str_pos=(char*)str;
	do{
		if('*'==*pat_pos){
			pat_pos++;
			while(*str_pos){
				if(*str_pos==*pat_pos){
					break;
				}else{
					str_pos++;
				}
			}
		}
		if(!*pat_pos){
			*len=(int)(str_pos-str);
			return (char*)str;
		}
		if(*str_pos==*pat_pos || '?'==*pat_pos){
			pat_pos++;
		}else{
			pat_pos=(char*)pat;
			str_pos=(char*)str++;
		}
	}while(*str_pos++);
	return NULL;
}


//ߕ̕
void instcpy(char*out,char*in){
	while(' '==*in || '\t'==*in) in++;//ŏ̃Xy[X菜
	while(*out=*in){
		if('\r'==*out || '\n'==*out || EOF==*out){//EOFAsR[h𖳎
			*out='\0';
			break;
		}

		//""ň͂܂ꂽ͏O
		if('\"'==*in){
			*out++=*in++;
			while('\"'!=*in){
				if('\0'==*in){*out='\0';setVariable("system.err","ERR1");return;}//"̖G[
#ifdef SHIFT_JIS
				//VtgJIS̓ǂݍ
				if((0x81<=(unsigned char)*in && 0x9F>=(unsigned char)*in) || (0xE0<=(unsigned char)*in && 0xFC>=(unsigned char)*in)){
					*out++=*in++;
					if('\0'==*in){*out='\0';setVariable("system.err","ERR2");break;}//vIȃG[
					*out++=*in++;
					if('\0'==*in){*out='\0';setVariable("system.err","ERR3");break;}//vIȃG[
				}else 
#endif
				if('\\'==*in){
					*out++=*in++;//GXP[vV[PX̓ǂݍ
					if('\0'==*in){*out='\0';setVariable("system.err","ERR4");return;}//"̖G[
					*out++=*in++;//GXP[vV[PX̓ǂݍ
					if('\0'==*in){*out='\0';setVariable("system.err","ERR5");return;}//"̖G[
				}else{
					*out++=*in++;
				}
			}
			*out=*in;
		}

		if('\''==*out){//'̓Rgƍl
			*out='\0';
			break;
		}

		if('\t'==*out){//tabXy[Xɒu
			*out=' ';
		}
		if((' '==*in || '\t'==*in) && (' '==*(in+1) || '\t'==*(in+1)))  out--;//AȂXy[X폜

		out++;in++;
	}
}


//s
void shell(char*str){

	str++;//擪菜
	trim_a(str,'\"');// "菜

#ifdef _WIN32 //gݍ݃}NɂVCG++̐؂蕪

//Windows
	STARTUPINFO si;// X^[gAbv
	PROCESS_INFORMATION pi;// vZX

	//STARTUPINFO \̂̓e擾 
	GetStartupInfo(&si);

	//ȂԂŋNɂ́A
	//si.dwFlags = STARTF_USESHOWWINDOW;
	//si.wShowWindow = SW_HIDE;

	if(0!=CreateProcess(
			NULL,					// s\W[̖O
			str,					// R}hC̕
			NULL,					// ZLeBLqq
			NULL,					// ZLeBLqq
			FALSE,					// nȟpIvV
			NULL,					// 쐬̃tO 
									// CREATE_NEW_PROCESS_GROUP	: VȃvZX
			NULL,					// VubN
			NULL,					// JgfBNg̖O
			&si,					// X^[gAbv
			&pi						// vZX
			)){

		//vZX̏Iҋ@
		CloseHandle(pi.hThread);
		WaitForSingleObject(pi.hProcess,INFINITE);
		CloseHandle(pi.hProcess);
	}else{
		//G[
		setVariable("system.err","ERR");
	}

#else
//Linux
	system(str);
#endif
}

//-------------------------------------------------------------------------------
